@@ -56,7 +56,7 @@
 #define	__G_UTILS_C__
 #include "glib.h"
 #include "gprintfint.h"
-#include "gthreadinit.h"
+#include "gthreadprivate.h"
 #include "galias.h"
 
 #ifdef	MAXPATHLEN
@@ -1399,6 +1399,13 @@ g_unsetenv (const gchar *variable)
  * Returns: a %NULL-terminated list of strings which must be freed
  * with g_strfreev().
  *
+ * Programs that want to be portable to Windows should typically use
+ * this function and g_getenv() instead of using the environ array
+ * from the C library directly. On Windows, the strings in the environ
+ * array are in system codepage encoding, while in most of the typical
+ * use cases for environment variables in GLib-using programs you want
+ * the UTF-8 encoding that this function and g_getenv() provide.
+ *
  * Since: 2.8
  */
 gchar **
