@@ -2204,11 +2204,16 @@ g_object_compat_control (gsize           what,
 {
   switch (what)
     {
+      gpointer *pp;
     case 1:     /* floating base type */
       return G_TYPE_INITIALLY_UNOWNED;
     case 2:     /* FIXME: remove this once GLib/Gtk+ break ABI again */
       floating_flag_handler = (guint(*)(GObject*,gint)) data;
       return 1;
+    case 3:     /* FIXME: remove this once GLib/Gtk+ break ABI again */
+      pp = data;
+      *pp = floating_flag_handler;
+      return 1;
     default:
       return 0;
     }
